import { Builder, By, WebDriver, until } from "selenium-webdriver";
import assert from "assert";
//import chrome from "selenium-webdriver/chrome";
import { Options as ChromeOptions } from 'selenium-webdriver/chrome';
import { Options as FirefoxOptions } from 'selenium-webdriver/firefox';
import dotenv from "dotenv";
dotenv.config() // read ".env"

describe("Login Test", function () {
  let driver:WebDriver;
  const BROWSER_NAME = process.env.BROWSER_NAME!;
  const SL_TEST_USE_REMOTE = process.env.SL_TEST_USE_REMOTE; 
  const SL_TEST_REMOTE_URL = process.env.SL_TEST_REMOTE_URL!;
  const SL_TEST_BASE_URL = process.env.SL_TEST_BASE_URL!;

  beforeEach(async function () {
    //console.log("Selenium Testing Server: " + process.env.APP_TESTS_REMOTE_WEB_DRIVER_URL);
    //driver = await new Builder().forBrowser("chrome").usingServer(`${process.env.APP_TESTS_REMOTE_WEB_DRIVER_URL}`).setChromeOptions(new chrome.Options().headless().windowSize({width: 1920, height: 1080})).build();
    //driver = await new Builder().forBrowser("chrome").setChromeOptions(new chrome.Options().headless()).build();
    //navigate to the application
    //await driver.get(`http://localhost:3000`);
    //await driver.manage().window().maximize();

    let builder = new Builder()
      .forBrowser(BROWSER_NAME)
      .setChromeOptions(new ChromeOptions().windowSize({ width: 1280, height: 720 }).headless())
      .setFirefoxOptions(new FirefoxOptions().windowSize({ width: 1280, height: 720 }).headless());
    if (SL_TEST_USE_REMOTE === "1") {
      builder = builder.usingServer(SL_TEST_REMOTE_URL);
    }
    driver = await builder.build();
    await driver.get(SL_TEST_BASE_URL);
    driver.manage().window().maximize();
  },10000);

  afterEach(async function () {
    await driver.quit();
  }); 

  it("login with correct credentials", async function () { 
    const loginButton = await driver.findElement(By.id("loginButton"));
    await loginButton.click();
    await driver.wait(until.elementLocated(By.id("loginDialog")), 5000);

    // Überprüfe, ob der Dialog geöffnet wurde
    const isDialogOpen = await driver.findElement(By.id("loginDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");
    
    const inputMail = await driver.findElement(By.id("inputEmail"));
    const inputPassword = await driver.findElement(By.id("inputPassword"));
    await inputMail.sendKeys("john@some-host.de");
    await inputPassword.sendKeys("12abcAB!");

    const okButton = await driver.findElement(By.id("okButton"));
    await okButton.click();   

    await driver.wait(until.elementLocated(By.id("logoutButton")), 5000);

    const isLogoutButtonOpen = await driver.findElement(By.id("logoutButton")).isDisplayed();
    assert.strictEqual(isLogoutButtonOpen, true, "Der Logout Button wird nicht angezeigt.");
  },10000);

  it("test login with wrong credentials", async function () {
    // test
    await driver.findElement(By.id("loginButton")).click();

    await driver.wait(until.elementLocated(By.id("loginDialog")), 5000);

    // Überprüfe, ob der Dialog geöffnet wurde
    const isDialogOpen = await driver.findElement(By.id("loginDialog")).isDisplayed();
    await driver.sleep(500);
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    // Falsche Anmeldeinformationen eingeben
    const inputMail = await driver.findElement(By.id("inputEmail"));
    const inputPassword = await driver.findElement(By.id("inputPassword"));
    await inputMail.sendKeys("john@some-host.de");
    await inputPassword.sendKeys("falschesPasswort");
    const okButton = await driver.findElement(By.id("okButton"));
    await okButton.click();

    await driver.wait(until.elementLocated(By.id("errorMessage")), 5000);

    // Überprüfen, ob eine Fehlermeldung oder ein Verhalten für falsche Anmeldeinformationen angezeigt wird
    const isErrorMessageVisible = await driver.findElement(By.id("errorMessage")).isDisplayed();
    assert.strictEqual(isErrorMessageVisible, true, "Falsche Anmeldeinformationen werden nicht erkannt.");
  },10000);

  it("user cant see user management", async function () {
    const loginButton = await driver.findElement(By.id("loginButton"));
    await loginButton.click();
    await driver.wait(until.elementLocated(By.id("loginDialog")), 5000);

    // Überprüfe, ob der Dialog geöffnet wurde
    const isDialogOpen = await driver.findElement(By.id("loginDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");
    
    const inputMail = await driver.findElement(By.id("inputEmail"));
    const inputPassword = await driver.findElement(By.id("inputPassword"));
    await inputMail.sendKeys("flo@some-host.de");
    await inputPassword.sendKeys("12abcAB!");

    const okButton = await driver.findElement(By.id("okButton"));
    await okButton.click();   

    await driver.wait(until.elementLocated(By.id("logoutButton")), 5000);

    const isLogoutButtonOpen = await driver.findElement(By.id("logoutButton")).isDisplayed();
    assert.strictEqual(isLogoutButtonOpen, true, "Der Logout Button wird nicht angezeigt.");

    //dont show user management
    const UserManagmentLink = await driver.findElements(By.linkText("UserManagment"));
    assert.strictEqual(UserManagmentLink.length, 0, "Der Link zu UserManagment sollte nicht sichtbar sein.");
  }, 10000)
});
